// naumann@stce.rwth-aachen.de
#include "Eigen/Dense"
#include "dco.hpp"
#include <iostream>
#include <cmath>
using namespace std;

template<typename T>
T f(const Eigen::VectorX<T>& x) {
  T y=0;
  for (int i=0;i<x.size();++i) {
    y=y+pow(x(i),2);
  }
  return sin(y);
}

template<typename T>
Eigen::VectorX<T> df_dx(Eigen::VectorX<T> x) {
  using AT=dco::ga1s<T>::type;
  dco::smart_tape_ptr_t<dco::mode<AT>> tape;
  Eigen::VectorX<AT> x_a(x.size());
  for (int i=0;i<x.size();++i) {
    x_a(i)=x(i);
    tape->register_variable(x_a(i));
  }
  AT y_a=f(x_a);
  dco::derivative(y_a)=1;
  tape->interpret_adjoint();
  Eigen::VectorX<T> dfdx(x.size());
  for (int i=0;i<x.size();++i) {
    dfdx(i)=dco::derivative(x_a(i));
  }
  return dfdx;
}

int main() {
  using T=double;
  int s;
  cout << "size="; cin >> s;
  Eigen::VectorX<T> x=Eigen::VectorX<T>::Random(s);
  cout << "f(x)=" << f(x) << endl;
  cout << "dfdx=" << df_dx(x) << endl;
  return 0;
}
