// naumann@stce.rwth-aachen.de

#include "Eigen/Dense"
#include "dco.hpp"

#include <cmath>
#include <iostream>
using namespace std;

template<typename T>
T f(const Eigen::VectorX<T>& x) {
  T y=0;
  for (int i=0;i<x.size();++i) {
    y=y+pow(x(i),2);
  }
  return sin(y);
}

template<typename T>
Eigen::VectorX<T> df_dx(Eigen::VectorX<T> x) {
  using TT=dco::gt1s<T>::type;
  Eigen::VectorX<TT> x_t(x.size());
  for (int i=0;i<x.size();++i) {
    x_t(i)=x(i);
  }
  Eigen::VectorX<T> dfdx(x.size());
  for (int i=0;i<x.size();++i) {
    dco::derivative(x_t(i))=1;
    TT y_t=f(x_t);
    dfdx(i)=dco::derivative(y_t);
    dco::derivative(x_t(i))=0;
  }
  return dfdx;
}

int main() {
  using T=double;
  int s;
  cout << "size="; cin >> s;
  Eigen::VectorX<T> x=Eigen::VectorX<T>::Random(s);
  cout << "f(x)=" << f(x) << endl;
  cout << "dfdx=" << df_dx(x) << endl;
  return 0;
}
